create database if not exists DPTW character set utf8mb4 collate utf8mb4_general_ci;
use DPTW;

delimiter ;^

	drop function if exists sessions_validate;^
	create function sessions_validate(
		$id integer 
	) returns bigint begin 
		return (case 
			when $id is null then 1 << 0 
			when $id < 0 then 1 << 1 
		else ifnull((
			select (case 
				when date_out is not null then 1 << 3 
				when timestampdiff(second, now(), date_last) > 30 then 1 << 4 
			else 0 end) from Sessions where id = $id limit 1 
		), 1 << 2) end);
	end;^
	
	drop function if exists sessions_exists;^
	create function sessions_exists(
		$id integer 
	) returns bool begin 
		return (select 1 from Sessions where id = $id limit 1) is not null;
	end;^
	
	drop procedure if exists procedures_get;^
	create procedure procedures_get(
		in `$database` varchar(64), 
		in `$procedure` varchar(64), 
		out $procedure_id integer 
	) begin 
		
		declare $database_id integer default (select id from `Databases` where date_out is null && name = `$database` limit 1);
	
		if $database_id is null then begin 
			insert into `Databases`(name) values(`$database`);
			set $database_id := last_insert_id();
		end;end if;
		
		set $procedure_id := (select id from Procedures where date_out is null && `database` = $database_id && name = `$procedure` limit 1);
	
		if $procedure_id is null then begin 
			insert into Procedures(`database`, name) values($database_id, `$procedure`);
			set $procedure_id := last_insert_id();
		end;end if;
		
	end;^
	
	drop procedure if exists parameters_get;^
	create procedure parameters_get(
		in $parameters text, 
		out $parameters_id integer
	) begin 
		
		set $parameters_id := (select id from Parameters where date_out is null && `data` = $parameters limit 1);
	
		if $parameters_id is null then begin 
			insert into Parameters(`data`) values($parameters);
			set $parameters_id := last_insert_id();
		end;end if;
		
	end;^
	
	drop procedure if exists logs_set;^
	create procedure logs_set(
		in `$session` integer, 
		in `$database` varchar(64), 
		in `$procedure` varchar(64), 
		in $parameters text, 
		in $error bigint, 
		in $register bool 
	) begin 
		if $register then begin 
		
			declare $procedure_id integer;
			declare $parameters_id integer;
		
			call procedures_get(`$database`, `$procedure`, $procedure_id);
			call parameters_get($parameters, $parameters_id);
		
			insert into Logs(`session`, `procedure`, parameters, error) values
				(if(sessions_exists(`$session`), `$session`, null), $procedure_id, $parameters_id, $error);
		
		end;end if;
	end;^
	
	drop procedure if exists exceptions_set;^
	create procedure exceptions_set(
		in `$session` integer, 
		in `$database` varchar(64), 
		in `$procedure` varchar(64), 
		in $parameters text, 
		in $error bigint, 
		in $message text, 
		in $status varchar(16), 
		in $code integer 
	) begin 
		
		declare $procedure_id integer;
		declare $parameters_id integer;
		declare $message_id integer default (select id from Messages where date_out is null && `text` = $message limit 1);
	
		call procedures_get(`$database`, `$procedure`, $procedure_id);
		call parameters_get($parameters, $parameters_id);
	
		if $message_id is null then begin 
			insert into Messages(`text`) values($message);
			set $message_id := last_insert_id();
		end;end if;
	
		insert into Exceptions(`session`, `procedure`, parameters, error, message, status, code) values
			(if(sessions_exists(`$session`), `$session`, null), $procedure_id, $parameters_id, $error, $message_id, $status, $code);
		
	end;^

delimiter ;